package com.lightinit.hsdataplatform.service.admin;

import com.lightinit.hsdataplatform.model.ResultState;
import com.lightinit.hsdataplatform.model.admin.DataPublishDetailModel;
import com.lightinit.hsdataplatform.model.admin.DataPublishListModel;
import com.lightinit.hsdataplatform.model.admin.DataPublishSearchModel;
import com.lightinit.hsdataplatform.model.admin.ResourceDataRegisterDataModel;

import java.util.List;

public interface IDataPublishMgrService {

    List<DataPublishListModel> queryPublishResourceDataList(DataPublishSearchModel inputModel);

    long queryCount(DataPublishSearchModel inputModel);

    ResultState<Long> publishResourceDetailAdd(DataPublishDetailModel dataPublishDetailModel);

    List<DataPublishListModel> queryList(DataPublishSearchModel inputModel);

    long queryListCount(DataPublishSearchModel inputModel);

    ResultState<Long> delete(long id);

    DataPublishDetailModel queryPublishInfoOne(Long id);

    ResultState<Long> updatePublishDataDetial(DataPublishDetailModel dataPublishDetailModel);

    ResultState offPublishData(Long id);
}
